@@ -1,4 +1,3 @@
-import dataclasses
 import re
 from itertools import chain
 from typing import (
@@ -34,7 +33,7 @@ from debputy.lsprotocol.types import (
     SemanticTokenTypes,
 )
 
-from debputy.linting.lint_util import LintState
+from debputy.linting.lint_util import LintState, te_position_to_lsp
 from debputy.lsp.debputy_ls import DebputyLanguageServer
 from debputy.lsp.lsp_debian_control_reference_data import (
     Deb822FileMetadata,
@@ -45,7 +44,6 @@ from debputy.lsp.lsp_debian_control_reference_data import (
 )
 from debputy.lsp.lsp_features import SEMANTIC_TOKEN_TYPES_IDS
 from debputy.lsp.text_util import (
-    te_position_to_lsp,
     trim_end_of_line_whitespace,
     SemanticTokensState,
 )
@@ -155,7 +153,7 @@ def _allow_stanza_continuation(
 
 def _at_cursor(
     deb822_file: Deb822FileElement,
-    file_metadata: Deb822FileMetadata[S],
+    file_metadata: Deb822FileMetadata[S, F],
     doc: "TextDocument",
     lines: List[str],
     client_position: Position,
@@ -241,7 +239,7 @@ def _at_cursor(
 def deb822_completer(
     ls: "DebputyLanguageServer",
     params: CompletionParams,
-    file_metadata: Deb822FileMetadata[Any],
+    file_metadata: Deb822FileMetadata[Any, Any],
 ) -> Optional[Union[CompletionList, Sequence[CompletionItem]]]:
     doc = ls.workspace.get_text_document(params.text_document.uri)
     lines = doc.lines
@@ -255,7 +253,7 @@ def deb822_completer(
         return None
 
     (
-        _a,
+        server_pos,
         current_field,
         word_at_position,
         in_value,
@@ -271,6 +269,9 @@ def deb822_completer(
         is_completion=True,
     )
 
+    if lines[server_pos.line].startswith("#"):
+        return
+
     items: Optional[Sequence[CompletionItem]]
     markdown_kind = ls.completion_item_document_markup(
         MarkupKind.Markdown, MarkupKind.PlainText
@@ -306,7 +307,7 @@ def deb822_completer(
 def deb822_hover(
     ls: "DebputyLanguageServer",
     params: HoverParams,
-    file_metadata: Deb822FileMetadata[S],
+    file_metadata: Deb822FileMetadata[S, F],
     *,
     custom_handler: Optional[
         Callable[
@@ -349,6 +350,10 @@ def deb822_hover(
         lines,
         params.position,
     )
+
+    if lines[server_pos.line].startswith("#"):
+        return
+
     hover_text = None
     if custom_handler is not None:
         res = custom_handler(
@@ -378,13 +383,19 @@ def deb822_hover(
             keyword = known_field.known_values.get(word_at_position)
             if keyword is None:
                 return None
-            hover_text = keyword.hover_text
+            hover_text = keyword.long_description_translated(ls)
             if hover_text is not None:
-                hover_text = f"# Value `{keyword.value}` (Field: {known_field.name})\n\n{hover_text}"
+                header = "`{VALUE}` (Field: {FIELD_NAME})".format(
+                    VALUE=keyword.value,
+                    FIELD_NAME=known_field.name,
+                )
+                hover_text = f"# {header}\n\n{hover_text}"
         else:
-            hover_text = known_field.hover_text
+            hover_text = known_field.long_description_translated(ls)
             if hover_text is None:
-                hover_text = f"The field {current_field} had no documentation."
+                hover_text = (
+                    f"No documentation is available for the field {current_field}."
+                )
             hover_text = f"# {known_field.name}\n\n{hover_text}"
 
     if hover_text is None:
@@ -426,7 +437,7 @@ def deb822_folding_ranges(
     ls: "DebputyLanguageServer",
     params: FoldingRangeParams,
     # Unused for now: might be relevant for supporting folding for some fields
-    _file_metadata: Deb822FileMetadata[Any],
+    _file_metadata: Deb822FileMetadata[Any, Any],
 ) -> Optional[Sequence[FoldingRange]]:
     doc = ls.workspace.get_text_document(params.text_document.uri)
     comment_start = -1
@@ -469,7 +480,7 @@ class Deb822SemanticTokensState(SemanticTokensState):
         doc: "TextDocument",
         lines: List[str],
         tokens: List[int],
-        file_metadata: Deb822FileMetadata[Any],
+        file_metadata: Deb822FileMetadata[Any, Any],
         keyword_token_code: int,
         known_value_token_code: int,
         comment_token_code: int,
@@ -567,7 +578,7 @@ def _deb822_paragraph_semantic_tokens_full(
 
 def deb822_format_file(
     lint_state: LintState,
-    file_metadata: Deb822FileMetadata[Any],
+    file_metadata: Deb822FileMetadata[Any, Any],
 ) -> Optional[Sequence[TextEdit]]:
     effective_preference = lint_state.effective_preference
     if effective_preference is None:
@@ -594,7 +605,7 @@ def deb822_format_file(
 def deb822_semantic_tokens_full(
     ls: "DebputyLanguageServer",
     request: SemanticTokensParams,
-    file_metadata: Deb822FileMetadata[Any],
+    file_metadata: Deb822FileMetadata[Any, Any],
 ) -> Optional[SemanticTokens]:
     doc = ls.workspace.get_text_document(request.text_document.uri)
     position_codec = doc.position_codec
@@ -645,15 +656,14 @@ def deb822_semantic_tokens_full(
 
 def _complete_field_name(
     lint_state: LintState,
-    fields: StanzaMetadata[Any],
+    stanza_metadata: StanzaMetadata[Any],
     matched_stanzas: Iterable[Deb822ParagraphElement],
     markdown_kind: MarkupKind,
 ) -> Sequence[CompletionItem]:
     items = []
     matched_stanzas = list(matched_stanzas)
-    # TODO: Normalize fields according to file rules (X[BCS]- should be stripped in some files)
     seen_fields = set(
-        f.lower()
+        stanza_metadata.normalize_field_name(f.lower())
         for f in chain.from_iterable(
             # The typing from python3-debian is not entirely optimal here. The iter always return a
             # `str`, but the provided type is `ParagraphKey` (because `__getitem__` supports those)
@@ -664,8 +674,8 @@ def _complete_field_name(
             for s in matched_stanzas
         )
     )
-    for cand_key, cand in fields.items():
-        if cand_key.lower() in seen_fields:
+    for cand_key, cand in stanza_metadata.items():
+        if stanza_metadata.normalize_field_name(cand_key.lower()) in seen_fields:
             continue
         item = cand.complete_field(lint_state, matched_stanzas, markdown_kind)
         if item is not None:
